@@ -66,8 +66,11 @@ pub use db_args::DbArgs;
 pub mod tl_data;
 pub use tl_data::{TlData, TlDataEntry};
 
+pub mod keysalt;
+pub use keysalt::{EncryptionType, KeySalt, KeySalts, SaltType};
+
 pub mod kadmin;
-pub use kadmin::{KAdmin, KAdminImpl};
+pub use kadmin::{KAdmin, KAdminApiVersion, KAdminImpl};
 
 pub mod sync;
 
