@@ -1,5 +1,8 @@
 //! Python bindings to libkadm5
 
+use std::{collections::HashSet, str::FromStr};
+
+use either::Either;
 use pyo3::{
     prelude::*,
     types::{PyDict, PyString, PyTuple},
@@ -8,7 +11,8 @@ use pyo3::{
 use crate::{
     db_args::DbArgs,
     error::Result,
-    kadmin::KAdminImpl,
+    kadmin::{KAdminApiVersion, KAdminImpl},
+    keysalt::{EncryptionType, KeySalt, KeySalts, SaltType},
     params::Params,
     policy::Policy,
     principal::Principal,
@@ -19,10 +23,15 @@ use crate::{
 #[pymodule(name = "_lib")]
 fn init(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
+    m.add_class::<KAdminApiVersion>()?;
     m.add_class::<Params>()?;
     m.add_class::<DbArgs>()?;
     m.add_class::<TlDataEntry>()?;
     m.add_class::<TlData>()?;
+    m.add_class::<EncryptionType>()?;
+    m.add_class::<SaltType>()?;
+    m.add_class::<KeySalt>()?;
+    m.add_class::<KeySalts>()?;
     m.add_class::<KAdmin>()?;
     m.add_class::<Principal>()?;
     m.add_class::<Policy>()?;
@@ -107,6 +116,52 @@ impl DbArgs {
     }
 }
 
+#[pymethods]
+impl EncryptionType {
+    #[new]
+    fn py_new(enctype: Either<i32, String>) -> Result<Self> {
+        Ok(match enctype {
+            Either::Left(i) => i.try_into()?,
+            Either::Right(s) => EncryptionType::from_str(&s)?,
+        })
+    }
+}
+
+#[pymethods]
+impl SaltType {
+    #[new]
+    #[pyo3(signature = (salttype = None))]
+    fn py_new(salttype: Option<Either<i32, String>>) -> Result<Self> {
+        Ok(match salttype {
+            None => Default::default(),
+            Some(salttype) => match salttype {
+                Either::Left(i) => i.try_into()?,
+                Either::Right(s) => SaltType::from_str(&s)?,
+            },
+        })
+    }
+}
+
+#[pymethods]
+impl KeySalt {
+    #[new]
+    #[pyo3(signature = (enctype, salttype = None))]
+    fn py_new(enctype: EncryptionType, salttype: Option<SaltType>) -> Self {
+        Self {
+            enctype,
+            salttype: salttype.unwrap_or_default(),
+        }
+    }
+}
+
+#[pymethods]
+impl KeySalts {
+    #[new]
+    fn py_new(keysalts: HashSet<KeySalt>) -> Self {
+        Self { keysalts }
+    }
+}
+
 #[pymethods]
 impl TlDataEntry {
     #[new]
@@ -127,7 +182,11 @@ impl TlData {
 }
 
 impl KAdmin {
-    fn py_get_builder(params: Option<Params>, db_args: Option<DbArgs>) -> KAdminBuilder {
+    fn py_get_builder(
+        params: Option<Params>,
+        db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
+    ) -> KAdminBuilder {
         let mut builder = KAdminBuilder::default();
         if let Some(params) = params {
             builder = builder.params(params);
@@ -135,6 +194,9 @@ impl KAdmin {
         if let Some(db_args) = db_args {
             builder = builder.db_args(db_args);
         }
+        if let Some(api_version) = api_version {
+            builder = builder.api_version(api_version);
+        }
         builder
     }
 }
@@ -216,6 +278,9 @@ impl KAdmin {
             if let Some(max_renewable_life) = kwargs.get_item("max_renewable_life")? {
                 builder = builder.max_renewable_life(max_renewable_life.extract()?);
             }
+            if let Some(allowed_keysalts) = kwargs.get_item("allowed_keysalts")? {
+                builder = builder.allowed_keysalts(allowed_keysalts.extract()?);
+            }
             if let Some(tl_data) = kwargs.get_item("tl_data")? {
                 builder = builder.tl_data(tl_data.extract::<TlData>()?);
             }
@@ -245,56 +310,64 @@ impl KAdmin {
 
     #[cfg(feature = "client")]
     #[staticmethod]
-    #[pyo3(name = "with_password", signature = (client_name, password, params=None, db_args=None))]
+    #[pyo3(name = "with_password", signature = (client_name, password, params=None, db_args=None, api_version=None))]
     fn py_with_password(
         client_name: &str,
         password: &str,
         params: Option<Params>,
         db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
     ) -> Result<Self> {
-        Self::py_get_builder(params, db_args).with_password(client_name, password)
+        Self::py_get_builder(params, db_args, api_version).with_password(client_name, password)
     }
 
     #[cfg(feature = "client")]
     #[staticmethod]
-    #[pyo3(name = "with_keytab", signature = (client_name=None, keytab=None, params=None, db_args=None))]
+    #[pyo3(name = "with_keytab", signature = (client_name=None, keytab=None, params=None, db_args=None, api_version=None))]
     fn py_with_keytab(
         client_name: Option<&str>,
         keytab: Option<&str>,
         params: Option<Params>,
         db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
     ) -> Result<Self> {
-        Self::py_get_builder(params, db_args).with_keytab(client_name, keytab)
+        Self::py_get_builder(params, db_args, api_version).with_keytab(client_name, keytab)
     }
 
     #[cfg(feature = "client")]
     #[staticmethod]
-    #[pyo3(name = "with_ccache", signature = (client_name=None, ccache_name=None, params=None, db_args=None))]
+    #[pyo3(name = "with_ccache", signature = (client_name=None, ccache_name=None, params=None, db_args=None, api_version=None))]
     fn py_with_ccache(
         client_name: Option<&str>,
         ccache_name: Option<&str>,
         params: Option<Params>,
         db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
     ) -> Result<Self> {
-        Self::py_get_builder(params, db_args).with_ccache(client_name, ccache_name)
+        Self::py_get_builder(params, db_args, api_version).with_ccache(client_name, ccache_name)
     }
 
     #[cfg(feature = "client")]
     #[staticmethod]
-    #[pyo3(name = "with_anonymous", signature = (client_name, params=None, db_args=None))]
+    #[pyo3(name = "with_anonymous", signature = (client_name, params=None, db_args=None, api_version=None))]
     fn py_with_anonymous(
         client_name: &str,
         params: Option<Params>,
         db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
     ) -> Result<Self> {
-        Self::py_get_builder(params, db_args).with_anonymous(client_name)
+        Self::py_get_builder(params, db_args, api_version).with_anonymous(client_name)
     }
 
     #[cfg(feature = "local")]
     #[staticmethod]
-    #[pyo3(name = "with_local", signature = (params=None, db_args=None))]
-    fn py_with_local(params: Option<Params>, db_args: Option<DbArgs>) -> Result<Self> {
-        Self::py_get_builder(params, db_args).with_local()
+    #[pyo3(name = "with_local", signature = (params=None, db_args=None, api_version=None))]
+    fn py_with_local(
+        params: Option<Params>,
+        db_args: Option<DbArgs>,
+        api_version: Option<KAdminApiVersion>,
+    ) -> Result<Self> {
+        Self::py_get_builder(params, db_args, api_version).with_local()
     }
 }
 
@@ -348,8 +421,11 @@ impl Policy {
             if let Some(max_renewable_life) = kwargs.get_item("max_renewable_life")? {
                 modifier = modifier.max_renewable_life(max_renewable_life.extract()?);
             }
+            if let Some(allowed_keysalts) = kwargs.get_item("allowed_keysalts")? {
+                modifier = modifier.allowed_keysalts(allowed_keysalts.extract()?);
+            }
             if let Some(tl_data) = kwargs.get_item("tl_data")? {
-                modifier = modifier.tl_data(tl_data.extract::<TlData>()?);
+                modifier = modifier.tl_data(tl_data.extract()?);
             }
             Ok(modifier.modify(kadmin)?)
         } else {
@@ -375,6 +451,15 @@ mod exceptions {
         m.add("PyKAdminException", m.py().get_type::<PyKAdminException>())?;
         m.add("KAdminException", m.py().get_type::<KAdminException>())?;
         m.add("KerberosException", m.py().get_type::<KerberosException>())?;
+        m.add(
+            "EncryptionTypeConversion",
+            m.py().get_type::<EncryptionTypeConversion>(),
+        )?;
+        m.add(
+            "SaltTypeConversion",
+            m.py().get_type::<SaltTypeConversion>(),
+        )?;
+        m.add("KerberosException", m.py().get_type::<KerberosException>())?;
         m.add(
             "NullPointerDereference",
             m.py().get_type::<NullPointerDereference>(),
@@ -421,6 +506,18 @@ mod exceptions {
             :ivar code: Kerberos error code
             :ivar origin_message: Kerberos error message
             "});
+    create_exception!(
+        exceptions,
+        EncryptionTypeConversion,
+        PyKAdminException,
+        "Failed to convert to encryption type"
+    );
+    create_exception!(
+        exceptions,
+        SaltTypeConversion,
+        PyKAdminException,
+        "Failed to convert to salt type"
+    );
     create_exception!(
         exceptions,
         NullPointerDereference,
@@ -487,6 +584,10 @@ mod exceptions {
                     KAdminException::new_err(error.to_string()),
                     Some((*code, message)),
                 ),
+                Error::EncryptionTypeConversion => {
+                    (EncryptionTypeConversion::new_err(error.to_string()), None)
+                }
+                Error::SaltTypeConversion => (SaltTypeConversion::new_err(error.to_string()), None),
                 Error::NullPointerDereference => {
                     (NullPointerDereference::new_err(error.to_string()), None)
                 }
