@@ -1,9 +1,14 @@
 from kadmin_local._lib import (
     DbArgs,
+    EncryptionType,
     KAdmin,
+    KAdminApiVersion,
+    KeySalt,
+    KeySalts,
     Params,
     Policy,
     Principal,
+    SaltType,
     TlData,
     TlDataEntry,
     __version__,
@@ -11,10 +16,15 @@ from kadmin_local._lib import (
 
 __all__ = (
     "DbArgs",
+    "EncryptionType",
     "KAdmin",
+    "KAdminApiVersion",
+    "KeySalt",
+    "KeySalts",
     "Params",
     "Policy",
     "Principal",
+    "SaltType",
     "TlData",
     "TlDataEntry",
     "__version__",
